import { Knex } from "knex";

type TableDefinition = {
  name: string;
  columns: Record<string, string>;
};

const requiredSchema: TableDefinition[] = [
  {
    name: "users",
    columns: {
      id: "uuid",
      email: "string",
      name: "string",
      attributes: "json"
    }
  },
  {
    name: "roles",
    columns: {
      id: "uuid",
      name: "string"
    }
  },
  {
    name: "user_roles",
    columns: {
      user_id: "uuid",
      role_id: "uuid"
    }
  },
  {
    name: "permissions",
    columns: {
      id: "uuid",
      resource: "string",
      action: "string",
      attributes_condition: "json"
    }
  },
  {
    name: "role_permissions",
    columns: {
      role_id: "uuid",
      permission_id: "uuid"
    }
  }
];

export async function validateSchema(knex: Knex) {
  for (const table of requiredSchema) {
    const exists = await knex.schema.hasTable(table.name);
    if (!exists) {
      throw new Error(`❌ Missing table: ${table.name}`);
    }

    const columns = await knex(table.name).columnInfo();
    for (const [col, expectedType] of Object.entries(table.columns)) {
      if (!columns[col]) {
        throw new Error(`❌ Table "${table.name}" is missing required column "${col}"`);
      }

      const actualType = columns[col].type;
      if (!actualType.includes(expectedType)) {
        console.warn(`⚠️ Column type mismatch in "${table.name}.${col}". Expected: "${expectedType}", got: "${actualType}"`);
      }
    }
  }

  console.log("✅ Permission schema validated successfully.");
}
